import os
import sys
from pydub import AudioSegment

def convert_mp3_to_wav(input_file, output_file=None):
    # 检查输入文件是否存在
    if not os.path.exists(input_file):
        print(f"Error: Input file '{input_file}' does not exist.")
        return

    # 如果没有指定输出文件名，则使用输入文件名（改变扩展名）
    if output_file is None:
        output_file = os.path.splitext(input_file)[0] + '.wav'

    try:
        # 读取MP3文件
        audio = AudioSegment.from_mp3(input_file)

        # 导出为WAV文件
        audio.export(output_file, format="wav")

        print(f"Conversion successful. WAV file saved as '{output_file}'")
    except Exception as e:
        print(f"An error occurred during conversion: {str(e)}")

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Usage: python convert_mp3_to_wav.py <input_mp3_file> [output_wav_file]")
    else:
        input_file = sys.argv[1]
        output_file = sys.argv[2] if len(sys.argv) > 2 else None
        convert_mp3_to_wav(input_file, output_file)
