from flask import Flask, request, send_file
import os
import sounddevice as sd
import numpy as np
import wave
import struct
from datetime import datetime

app = Flask(__name__)

# 配置
UPLOAD_FOLDER = 'uploads'
RESPONSE_FILE = '148.wav'  # 固定的响应文件
SAMPLE_RATE = 16000
SAMPLE_WIDTH = 2  # 16位音频
CHANNELS = 1

if not os.path.exists(UPLOAD_FOLDER):
    os.makedirs(UPLOAD_FOLDER)

def add_wav_header(audio_data, sample_rate=SAMPLE_RATE, bits_per_sample=SAMPLE_WIDTH*8, channels=CHANNELS):
    data_size = len(audio_data)
    total_size = data_size + 36
    header = struct.pack('<4sI4s4sIHHIIHH4sI',
                        b'RIFF', total_size, b'WAVE', b'fmt ', 16,
                        1, channels, sample_rate, sample_rate * channels * bits_per_sample // 8,
                        channels * bits_per_sample // 8, bits_per_sample, b'data', data_size)
    return header + audio_data

@app.route('/uploadAudio', methods=['POST'])
def upload_audio():
    print("Received a POST request")
    print("Content-Type:", request.content_type)
    print("Request data length:", request.content_length)

    if request.content_type == 'application/octet-stream':
        raw_audio = request.data
        wav_audio = add_wav_header(raw_audio)

        # 生成唯一的文件名
        timestamp = datetime.now().strftime('%Y%m%d_%H%M%S_%f')
        filename = os.path.join(UPLOAD_FOLDER, f'received_audio_{timestamp}.wav')

        with open(filename, 'wb') as f:
            f.write(wav_audio)
        print(f"File saved as {filename}")

        # 播放最新上传的音频（可选，用于调试）
        play_audio(filename)

        # 返回固定的响应文件
        if os.path.exists(RESPONSE_FILE):
            print(f"Sending response file: {RESPONSE_FILE}")
            return send_file(RESPONSE_FILE, mimetype='audio/wav')
        else:
            print(f"Error: Response file {RESPONSE_FILE} not found")
            return "Response file not found", 404
    else:
        print("Unexpected content type")
        return "Invalid content type", 400

def play_audio(filename):
    try:
        with wave.open(filename, 'rb') as wf:
            # 读取音频数据
            data = wf.readframes(wf.getnframes())
            audio_data = np.frombuffer(data, dtype=np.int16)

            # 播放音频
            sd.play(audio_data, SAMPLE_RATE)
            sd.wait()
    except Exception as e:
        print(f"Error playing audio: {e}")

if __name__ == '__main__':
    if not os.path.exists(RESPONSE_FILE):
        print(f"Warning: Response file {RESPONSE_FILE} does not exist!")
    else:
        print(f"Response file {RESPONSE_FILE} is ready, size: {os.path.getsize(RESPONSE_FILE)} bytes")
    app.run(host='0.0.0.0', port=8888, debug=True)