from flask import Flask, request
from pydub import AudioSegment
import io
import os
import time

app = Flask(__name__)

@app.route('/upload_audio', methods=['POST'])
def upload_audio():
    try:
        if 'Content-Type' not in request.headers or request.headers['Content-Type'] != 'application/octet-stream':
            return 'Invalid Content-Type', 400

        raw_audio = request.data
        
        # 将原始音频数据转换为AudioSegment对象
        audio = AudioSegment(
            data=raw_audio,
            sample_width=2,  # 16-bit audio
            frame_rate=16000,
            channels=1
        )

        # 生成唯一的文件名
        filename = f"audio_{int(time.time())}.mp3"
        
        # 导出为MP3
        audio.export(filename, format="mp3")

        print(f"Audio saved as {filename}")
        return f'Audio saved as {filename}', 200
    except Exception as e:
        print(f"Error processing audio: {str(e)}")
        return f'Error processing audio: {str(e)}', 500

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000, debug=True)