import socket
import pyaudio
import numpy as np
import time

# UDP设置
UDP_IP = "0.0.0.0"  # 监听所有可用的网络接口
UDP_PORT = 3333
ESP32_IP = "192.168.0.173"  # ESP32-S3的IP地址

# 音频设置
FORMAT = pyaudio.paInt16
CHANNELS = 1
RATE = 16000
CHUNK = 1024

# 初始化PyAudio
p = pyaudio.PyAudio()
stream = p.open(format=FORMAT,
                channels=CHANNELS,
                rate=RATE,
                output=True,
                frames_per_buffer=CHUNK)

# 创建UDP socket
sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
sock.bind((UDP_IP, UDP_PORT))
sock.settimeout(1.0)  # 设置1秒超时

print(f"Listening for audio on port {UDP_PORT}...")
print(f"Expecting data from ESP32-S3 at IP {ESP32_IP}")

packets_received = 0
last_debug_time = time.time()

try:
    while True:
        try:
            data, addr = sock.recvfrom(CHUNK * 2)  # 因为每个样本是16位的
            if addr[0] == ESP32_IP:
                audio_data = np.frombuffer(data, dtype=np.int16)
                stream.write(audio_data.tobytes())
                packets_received += 1
            else:
                print(f"Received data from unexpected IP: {addr[0]}")
        except socket.timeout:
            print("No data received in the last second")

        current_time = time.time()
        if current_time - last_debug_time >= 1:
            print(f"Packets received in the last second: {packets_received}")
            packets_received = 0
            last_debug_time = current_time

except KeyboardInterrupt:
    print("Stopping...")
finally:
    stream.stop_stream()
    stream.close()
    p.terminate()
    sock.close()