from openai import OpenAI
from pydub import AudioSegment
from pydub.playback import play

class OpenAIAudioTranscription:
    def __init__(self, api_key, base_url):
        self.client = OpenAI(
            api_key=api_key,
            base_url=base_url
        )

    def transcribe(self, audio_file_path):
        """
        Transcribe the given audio file to text.
        Supports both MP3 and WAV formats.
        """
        if audio_file_path.endswith(".mp3"):
            audio = AudioSegment.from_mp3(audio_file_path)
        elif audio_file_path.endswith(".wav"):
            audio = AudioSegment.from_wav(audio_file_path)
        else:
            raise ValueError("Unsupported audio file format. Please provide a .mp3 or .wav file.")

        with open("temp.mp3", "wb") as temp_file:
            audio.export(temp_file, format="mp3")

        with open("temp.mp3", "rb") as audio_file:
            transcription = self.client.audio.transcriptions.create(
                model="whisper-1",
                file=audio_file
            )

        return transcription.text

    def answer_voice_question(self, audio_file_path):
        """
        Transcribe the given audio file, then use GPT-3.5-Turbo to answer the question and generate an audio response.
        """
        transcript = self.transcribe(audio_file_path)
        print(f"Transcript: {transcript}")

        response = self.client.chat.completions.create(
            model="gpt-3.5-turbo",
            messages=[
                {"role": "system", "content": "You are a helpful assistant."},
                {"role": "user", "content": transcript}
            ],
            stream=True
        )

        audio_chunks = []
        for chunk in response:
            audio_chunk = self.client.audio.generate(
                text=chunk.choices[0].text,
                model="whisper-1"
            )
            audio_chunks.append(audio_chunk)

        # Combine the audio chunks and save the result
        result_audio = AudioSegment.silent(duration=0)
        for chunk in audio_chunks:
            with open("temp.mp3", "wb") as temp_file:
                chunk.export(temp_file, format="mp3")
            result_audio += AudioSegment.from_mp3("temp.mp3")

        result_audio_path = "result.mp3"
        result_audio.export(result_audio_path, format="mp3")
        return result_audio_path

# Usage example
transcription = OpenAIAudioTranscription(
    api_key="sk-HRko1EyfU8TrMSSs0dAcB5499279427c90616665D7F0783a",
    base_url="https://api.gpts.vin/v1"
)

while True:
    # audio_file_path = input("Please provide the path to your audio file: ")
    audio_file_path = "/media/w0x7ce/6912ab14-a241-47f9-b4b7-8278c4ef3d21/Desktop/esp32_s3_microphone_project/python_server/en.wav"
    result_audio_path = transcription.answer_voice_question(/media/w0x7ce/6912ab14-a241-47f9-b4b7-8278c4ef3d21/Desktop/esp32_s3_microphone_project/python_server/en.wav
    print(f"OpenAI's voice response saved to: {result_audio_path}")
    play(AudioSegment.from_mp3(result_audio_path))