import asyncio
import websockets
import wave
import struct

# WebSocket服务器配置
HOST = '0.0.0.0'  # 监听所有网络接口
PORT = 8888

# 音频配置
CHANNELS = 1
SAMPLE_WIDTH = 2  # 16位音频
SAMPLE_RATE = 44100

# 创建WAV文件
wav_file = wave.open('recorded_audio.wav', 'wb')
wav_file.setnchannels(CHANNELS)
wav_file.setsampwidth(SAMPLE_WIDTH)
wav_file.setframerate(SAMPLE_RATE)

async def audio_server(websocket, path):
    print(f"New connection from {websocket.remote_address}")
    try:
        async for message in websocket:
            # 假设接收到的是16位整数音频数据
            audio_data = struct.unpack(f'<{len(message)//2}h', message)
            wav_file.writeframes(struct.pack(f'<{len(audio_data)}h', *audio_data))
    except websockets.exceptions.ConnectionClosed:
        print(f"Connection closed from {websocket.remote_address}")
    finally:
        # 如果连接关闭,确保WAV文件被正确保存
        wav_file.close()
        print("WAV file saved")

async def main():
    server = await websockets.serve(audio_server, HOST, PORT)
    print(f"WebSocket server started on ws://{HOST}:{PORT}")
    await server.wait_closed()

if __name__ == "__main__":
    asyncio.run(main())
