import asyncio
import websockets
import wave
import struct
import sounddevice as sd
import numpy as np

# WebSocket服务器配置
HOST = '0.0.0.0'  # 监听所有网络接口
PORT = 8888

# 音频配置
CHANNELS = 1
SAMPLE_WIDTH = 2  # 16位音频
SAMPLE_RATE = 16000
BUFFER_LENGTH = 1024
CHUNK = BUFFER_LENGTH * 4  # 每次播放的块大小

# 创建WAV文件
wav_file = wave.open('recorded_audio.wav', 'wb')
wav_file.setnchannels(CHANNELS)
wav_file.setsampwidth(SAMPLE_WIDTH)
wav_file.setframerate(SAMPLE_RATE)

# 创建音频流
stream = sd.OutputStream(samplerate=SAMPLE_RATE, channels=CHANNELS, dtype='int16')
stream.start()

# 用于存储音频数据的队列
audio_queue = asyncio.Queue()

async def audio_processor():
    while True:
        audio_chunk = await audio_queue.get()
        wav_file.writeframes(audio_chunk)
        audio_data = np.frombuffer(audio_chunk, dtype=np.int16)
        stream.write(audio_data)

async def audio_server(websocket, path):
    print(f"New connection from {websocket.remote_address}")
    try:
        async for message in websocket:
            audio_data = struct.unpack(f'<{len(message)//2}h', message)
            audio_chunk = struct.pack(f'<{len(audio_data)}h', *audio_data)
            await audio_queue.put(audio_chunk)
    except websockets.exceptions.ConnectionClosed:
        print(f"Connection closed from {websocket.remote_address}")

async def main():
    server = await websockets.serve(audio_server, HOST, PORT)
    print(f"WebSocket server started on ws://{HOST}:{PORT}")
    
    # 启动音频处理任务
    asyncio.create_task(audio_processor())
    
    await server.wait_closed()

if __name__ == "__main__":
    try:
        asyncio.run(main())
    finally:
        wav_file.close()
        print("WAV file saved")
        stream.stop()
        stream.close()